"""Operator Module."""
from qutrunk.circuit.gates.basicgate import BasicGate


class Operator(BasicGate):
    """A series of quantum gate operations \
        provide a simple and convenient way to construct a quantum algorithm."""

    pass

