#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import sys
from .ttypes import *
ErrMsg = {
    4: "异常",
    2: "不合法参数",
    6: "存在",
    7: "缓存中",
    8: "内存不足",
    5: "不存在",
    9: "未初始化",
    10: "机器未注册",
    11: "无可用的资源",
    1: "未知错误",
    12: "数据解析异常",
    0: "成功",
    3: "超时",
}
